<?php

class YResilience_API {

  private $website_id = null;
  private $api_key = null;
  private $api_url = "api.yavli.com/v2";

  static $instance = false;

  public static function init() {
    if ( ! self::$instance ) {
      self::$instance = new YResilience_API;
    }
    return self::$instance;
  }

  function __construct() {

    $this->api_result = null;
    $this->api_status = null;
    $this->api_reachable = null;
    $this->api_error_code = null;
    $this->api_error_detail = null;

  }

  private function get_api_url($path, $use_ssl=true) {

    if ($use_ssl)
      return "https://" . $this->api_url . $path;
    else
      return "http://" . $this->api_url . $path;

  }

  private function get_auth_headers() {

    return array(
      "X-WebsiteID" => YResilience_Variables::variable_get( 'website_id' ),
      "X-ApiKey" => YResilience_Variables::variable_get( 'api_key' ),
      "X-PluginVersion" => YRESILIENCE_VERSION
    );

  }

  protected function api_request($action, $custom_args=array()) {

    $url = $this->get_api_url($action, false);
    $url_ssl = $this->get_api_url($action);

    $default_args = array(
      'headers'     => $this->get_auth_headers(),
      'method'      => "GET",
      'max_redirects' => 5,
      'timeout'     => 5,
      'retry'       => 3,
      'vars'        => array(),
    );
    $args = array_merge($default_args, $custom_args);

    if (6 === DRUPAL_VERSION) {

      $response = drupal_http_request($url_ssl, $args["headers"], $args["method"], http_build_query($args["vars"]), $args["retry"], $args["timeout"]);
      if (isset($response->error)) {
        $response = drupal_http_request($url, $args["headers"], $args["method"], http_build_query($args["vars"]), $args["retry"], $args["timeout"]);
      }
      $response = $response->data;

    } elseif (7 === DRUPAL_VERSION) {

      $response = drupal_http_request(
        $url_ssl,
        array(
          'headers' => $args["headers"],
          'method' => $args["method"],
          'data' => http_build_query($args["vars"]),
          'max_redirects' => $args["max_redirects"],
          'timeout' => $args["timeout"],
        )
      );

      if (isset($response->error)) {
        $response = drupal_http_request(
          $url,
          array(
            'headers' => $args["headers"],
            'method' => $args["method"],
            'data' => http_build_query($args["vars"]),
            'max_redirects' => $args["max_redirects"],
            'timeout' => $args["timeout"],
          )
        );
      }

      $response = $response->data;

    } elseif (8 === DRUPAL_VERSION) {

      try {

        $request = Drupal::httpClient()->createRequest(
          $args["method"],
          $url,
          array(
            'headers' => $args["headers"],
            'body' => http_build_query($args["vars"])
          )
        );
        $response = Drupal::httpClient()->send($request)->getBody();

      } catch (Exception $exception) {
        // drupal_set_message(t('Failed to complete api_request. Check Website ID and API Key.'), 'error');
      }

    }

    $result = json_decode($response);

    if ($result != null) {

      $this->api_reachable = true;

      if (property_exists($result, "error")) {
        $this->api_status = $result->error->status;
        $this->api_error_code = $result->error->code;
        $this->api_error_detail = $result->error->detail;
      } else {
        $this->api_status = 200;
        $this->api_result = $result;
      }

    } else {
      $this->api_reachable = false;
    }
  }

  public function get_ad_server() {

    $this->api_request("/get-ad-server");
    if ($this->api_reachable == true && $this->api_status == 200) {
      return true;
    }
    return false;

  }

  public function get_full_error_msg() {

    $msg = "API Reachable: ";
    $msg .= ($this->api_reachable ? "Yes" : "No");
    $msg .= ", API Status: '".$this->api_status."', ";
    $msg .= "API Error Code: '".$this->api_error_code."', ";
    $msg .= "API Error Detail: '".$this->api_error_detail."'";
    return $msg;

  }

}
